<?php
/**
 * Created by PhpStorm.
 * User: roc
 * Date: 2019/9/12
 * Time: 14:35
 */

namespace kwy\kwy_third_merchant_sdk\Wx;

use kwy\kwy_third_merchant_sdk\Common\Util;

class WxMchAuth
{
    private $oss_url = 'http://weiwoju.oss-cn-hangzhou.aliyuncs.com/uploads/';

    /**
     * 微信证书秘钥等配置
     * @var array
     */
    public $wxMchConf = [
        '235178645' => [    //乐刷渠道号
            'serial_no' => '333CDCD245B170DB422638D43CA31E5234144F73',  //本地证书序列号
            'kwy_key_path' => '/Common/Conf/wx/235178645/ca/apiclient_key.pem', //客无忧证书私钥文件路径
            'kwy_cert_path' => '/Common/Conf/wx/235178645/ca/apiclient_cert.pem',   //客无忧证书文件路径
            'wechatpay_serial' => '7EC04B12DF35242489EA5FF38E15C014EA358A4C',   //微信支付平台序列号
            'wx_public_cert' => '/Common/Conf/wx/235178645/ca/wx_public_cert.pem',  //微信支付平台证书文件路径
        ],
        '1272323401' => [    //乐刷渠道号
            'serial_no' => '5847068E66949014E868ED29D42C73D4B2DFBFF2',  //本地证书序列号
            'kwy_key_path' => '/ThinkPHP/Library/Vendor/WxPayV3-5/cert/1272323401/20180731/1272323401_20180731_key.pem', //客无忧证书私钥文件路径
            'kwy_cert_path' => '/ThinkPHP/Library/Vendor/WxPayV3-5/cert/1272323401/20180731/1272323401_20180731_cert.pem',   //客无忧证书文件路径
            'wechatpay_serial' => '3DE9ED900DD3B2E46F6B02AACB0433DD0DB55BD0',   //微信支付平台序列号
            'wx_public_cert' => '/ThinkPHP/Library/Vendor/WxPayV3-5/cert/1272323401/20180731/1272323401_20180731_public_key.pem',  //微信支付平台证书文件路径
        ]

    ];

    private $mch_id = '';   //微信渠道号/服务商号
    private $serial_no=''; //本地证书序列号
    private $channel_key_path=''; //服务商证书私钥文件路径
    private $channel_cert_path=''; //服务商证书文件路径
    private $wechat_pay_serial=''; //微信支付平台序列号
    private $wechat_public_cert=''; //微信支付平台证书文件路径

    private $log_info = ''; //日志文件

    /**
     * 设置服务商微信渠道号
     * @param $mch_id
     */
    public function setMch_id($mch_id){
        $this->mch_id = $mch_id;
    }

    /**
     * 本地证书序列号
     * @param $serial_no
     */
    public function setSerial_no($serial_no){
        $this->serial_no = $serial_no;
    }

    /**
     * 设置服务商证书私钥文件路径
     * @param $channel_key_path
     */
    public function setChannel_key_path($channel_key_path){
        $this->channel_key_path = $channel_key_path;
    }

    /**
     * 设置服务商证书文件路径
     * @param $channel_cert_path
     */
    public function setChannel_cert_path($channel_cert_path){
        $this->channel_cert_path = $channel_cert_path;
    }

    /**
     * 设置微信支付平台序列号
     * @param $wechat_pay_serial
     */
    public function setWechatpay_serial($wechat_pay_serial){
        $this->wechat_pay_serial = $wechat_pay_serial;
    }

    /**
     * 微信支付平台证书文件路径
     * @param $wechat_public_cert
     */
    public function setWechat_public_cert($wechat_public_cert){
        $this->wechat_public_cert = $wechat_public_cert;
    }

    /**
     * 提交申请单
     * @param $business_code
     * @param array $contactArr
     * @param array $identificationArr
     * @param array $subjectArr
     * @return mixed
     * @throws \Exception
     */
    public function applyMch($business_code,$contactArr=[],$identificationArr=[],$subjectArr=[]){
        $data = [
            'business_code' => $business_code, //商户旗下门店的唯一编号
            'contact_info' => $contactArr,    //联系人信息
            'identification_info' => $identificationArr,  //法人身份信息
            'subject_info' => $subjectArr,  //主体信息
        ];

        $url = 'https://api.mch.weixin.qq.com/v3/apply4subject/applyment';
        $result = $this->post($data,$url);
        return $result;
        ##################同步支付end#####################
    }

    /**
     * 撤销申请单
     * @param $applyment_id 撤销申请单
     * @return mixed
     * @throws \Exception
     */
    public function cancelApplyment($applyment_id){
        $url = "https://api.mch.weixin.qq.com/v3/apply4subject/applyment/{$applyment_id}/cancel";
        $data = [];
        $result = $this->post($data,$url);
        return $result;
        ##################同步支付end#####################
    }

    /**
     * 商户查询
     * @param $mch_id
     * @throws \Exception
     * @return mixed
     */
    public static function queryWxMch($mch_id){
        $url = "https://api.mch.weixin.qq.com/v3/apply4subject/applyment/merchants/{$mch_id}/state";
        $wxMchAuth = new self();
        return $wxMchAuth->get($url);
    }

    /**
     * 查询商户审核结果
     * @param $apply_id
     * @throws \Exception
     * @return mixed
     */
    public function queryApply($apply_id){
        $url = "https://api.mch.weixin.qq.com/v3/apply4subject/applyment?applyment_id={$apply_id}";
        return $this->get($url);
    }

    /**
     * 获取证书时的签名
     * @param $url
     * @param string $method
     * @param string $json_data
     * @return string
     * @throws \Exception
     */
    private function getAuthorization($url,$method='POST',$json_data=''){
        //第一步，获取 HTTP 请求的方法(GET,POST,PUT 等)

        //第二步，获取请求的 URL，并去除域名的部分，如果链接带参数，参数值必须进行 URLencode。
        $tmp_arr = explode('//',$url);
        $arr = explode('/',$tmp_arr[1]);
        unset($arr[0]);
        $url = '/'.implode('/',$arr);

        //第三步，获取发起请求时的系统当前时间戳，即格林威治时间1970年01月01日00时00分00秒 (北京时间1970年01月01日08时00分00秒)起至现在的总秒数，作为请求时间戳。时间戳必须是最新的，如果时间戳比微信支付服务器时间晚300秒，微信支付服务器会不认这个请求并报错，请商户保持自身系统的时间准确。
        $timestamp = time();

        //第四步，生成一个请求随机串，算法可开发者自定义(可调用系统随机数生成函数转化成字符串)，建议长度不少于 10 位。
        $nonce_str = Util::getNonceStr(32);


        if($method == 'GET'){
            $str = "{$method}\n{$url}\n{$timestamp}\n{$nonce_str}\n\n";
        }else{
            $str = $method."\n".$url."\n".$timestamp."\n".$nonce_str."\n".$json_data."\n";
        }
        //获取本地序列号
        $serial_no = $this->getKwySerial();

        $sign = $this->getPrivateEncrypt($str);
        $auth = 'mchid="'.$this->mch_id.'",nonce_str="'.$nonce_str.'",signature="'.$sign.'",timestamp="'.$timestamp.'",serial_no="'.$serial_no.'"';


        return 'WECHATPAY2-SHA256-RSA2048 '.$auth;
    }

    /**
     * 认证header头部信息加密
     * @param $str
     * @return string
     * @throws \Exception
     */
    private function getPrivateEncrypt($str){
        $private_key_path = getcwd().$this->getChannelKeyPath();
        $private_key = file_get_contents($private_key_path);
        $pr_key =  openssl_pkey_get_private($private_key);
        $binary_signature = "";
        openssl_sign($str, $binary_signature, $pr_key, OPENSSL_ALGO_SHA256);
        $sign = base64_encode($binary_signature);
        return $sign;
    }

    /**
     * 新版签名证书的post curl请求
     * @param $post_data
     * @param string $url
     * @return mixed
     * @throws \Exception
     */
    public function post($post_data,$url=''){
        //判断是否传入数据，没有的话，传入空串，否则转json对象
        if(!empty($post_data)){
            $json_data = json_encode($post_data,JSON_UNESCAPED_UNICODE);
        }else{
            $json_data = '';
        }
        $ch = curl_init();
        $header=[];
        $header[] = "Accept: application/json";
        $header[] = "Accept-Language: zh-CN";
        $header[] = "Content-Type: application/json";
        $header[] = "User-Agent: curl/7.54.0";
        $header[] = "Content-Length: ".strlen($json_data);
        $header[] = "Wechatpay-Serial: ".$this->getWechatpaySerial();
        $header[] = 'Authorization:'.$this->getAuthorization($url,'POST',$json_data);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url);//url
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);//返回结果curl
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );//强制ipv4解析
        curl_setopt($ch, CURLOPT_TIMEOUT, 60); //超时时间
        curl_setopt($ch,CURLOPT_HTTP_VERSION,CURL_HTTP_VERSION_1_1);      //(默认值，让cURL自己判断使用哪个版本)，CURL_HTTP_VERSION_1_0 (强制使用 HTTP/1.0)或CURL_HTTP_VERSION_1_1 (强制使用 HTTP/1.1)


        //设置证书
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,1);      //// 验证服务器证书有效性
        //使用证书：cert 与 key 分别属于两个.pem文件
        $cert_path = getcwd().$this->getChannelCertPath();
        $key_path = getcwd().$this->getChannelKeyPath();
        curl_setopt($ch,CURLOPT_SSLCERT, $cert_path);   //客户端证书路径
        curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM'); //证书的类型。支持的格式有"PEM" (默认值), "DER"和"ENG"
        curl_setopt($ch,CURLOPT_SSLKEY, $key_path); //客户端私钥的文件路径
        curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');  //客户端私钥类型，支持的私钥类型为"PEM"(默认值)、"DER"和"ENG"

        curl_setopt($ch, CURLOPT_POSTFIELDS, $json_data);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_HEADER, 1); //返回response头部信息
        curl_setopt($ch, CURLINFO_HEADER_OUT, true); //TRUE 时追踪句柄的请求字符串，从 PHP 5.1.3 开始可用。这个很关键，就是允许你查看请求header
        $ret = curl_exec($ch);

        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        $this->log_info = "微信实名认证post请求,：请求地址：{$url}====请求数据：".$json_data."====返回数据：".$ret;

        if($code == '204'){
            return true;
        }if($code == '200'){
            $arr = explode("\r\n\r\n",$ret);
            return end($arr);
        }else{
            $arr = explode("\r\n\r\n",$ret);
            throw new \Exception(end($arr));

            if(count($arr) != '2'){
                throw new \Exception('微信返回数据异常');
            }else{
                throw new \Exception($arr[1]);
                $data = json_decode($arr[1],true);
                throw new \Exception($data['message']);
            }
        }
        return false;
    }

    /**
     * 新版签名证书的post curl请求
     * @param string $url
     * @return mixed
     * @throws \Exception
     */
    public function get($url=''){
        $ch=curl_init();
        $header=[];
        $header[] = "Accept: application/json";
        $header[] = "Accept-Language: zh-CN";
        $header[] = "Content-Type: application/json";
        $header[] = "User-Agent: curl/7.54.0";
        $header[] = "Content-Length: 0";
        $header[] = 'Authorization:'.$this->getAuthorization($url,'GET');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url);//url
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);//返回结果curl
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );//强制ipv4解析
        curl_setopt($ch, CURLOPT_TIMEOUT, 60); //超时时间
        curl_setopt($ch,CURLOPT_HTTP_VERSION,CURL_HTTP_VERSION_1_1);      //(默认值，让cURL自己判断使用哪个版本)，CURL_HTTP_VERSION_1_0 (强制使用 HTTP/1.0)或CURL_HTTP_VERSION_1_1 (强制使用 HTTP/1.1)

        //设置证书
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,1);      //// 验证服务器证书有效性
        //使用证书：cert 与 key 分别属于两个.pem文件
        $cert_path = getcwd().$this->getChannelCertPath();
        $key_path = getcwd().$this->getChannelKeyPath();
        curl_setopt($ch,CURLOPT_SSLCERT, $cert_path);   //客户端证书路径
        curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM'); //证书的类型。支持的格式有"PEM" (默认值), "DER"和"ENG"
        curl_setopt($ch,CURLOPT_SSLKEY, $key_path); //客户端私钥的文件路径
        curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');  //客户端私钥类型，支持的私钥类型为"PEM"(默认值)、"DER"和"ENG"

        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
        curl_setopt($ch, CURLOPT_HEADER, 1); //返回response头部信息
        curl_setopt($ch, CURLINFO_HEADER_OUT, true); //TRUE 时追踪句柄的请求字符串，从 PHP 5.1.3 开始可用。这个很关键，就是允许你查看请求header
        $ret = curl_exec($ch);

        $this->log_info = "微信实名认证get请求：get_url：".$url."====返回数据：".$ret;

//        $info = curl_getinfo($ch);  //获取一个cURL连接资源句柄的信息;http://www.php.net/manual/zh/function.curl-getinfo.php
//        echo curl_getinfo($ch, CURLINFO_HEADER_OUT);    //查看请求header信息
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if($code == '204'){
            return true;
        }if($code != '200'){
            throw new \Exception('请求微信返回code状态异常');
        }else{
            $arr = explode("\r\n\r\n",$ret);
            /*if(count($arr) != '2'){
                throw new \Exception('微信返回数据异常');
            }else{
                return $arr[1];
            }*/
            return end($arr);
        }
    }

    /**
     * 私密信息加密
     * @param $str
     * @return string
     * @throws \Exception
     */
    public function getEncrypt($str){
        $public_cert_path = getcwd().$this->getWechatPublicCert();
        $public_key = file_get_contents($public_cert_path);
        $encrypted = '';
        openssl_public_encrypt($str,$encrypted,$public_key,OPENSSL_PKCS1_OAEP_PADDING);
        //base64编码
        $sign = base64_encode($encrypted);
        return $sign;
    }

    /**
     * 微信图片上传
     * @param $path
     * @param $name
     * @throws \Exception
     * @return mixed
     */
    public function uploadWxImg($path,$name=''){
        throw new \Exception('暂未实现');
        if (!is_dir('./Public/uploads/downloadimg')) {
            mkdir('./Public/uploads/downloadimg', 0777, true);
        }
        $path = str_replace($this->oss_url, '', $path);
        if(!empty($path)){
            $path = $this->oss_url.$path;
        }else{
            throw new \Exception($name.'图片不存在');
        }

        $pic = Util::download($path,true);
        if(filesize($pic) > 2048000){
            $original_pic = $pic;
            $pic = Util::get_thumb_img($original_pic,2048);
            @unlink($original_pic);
        }
        $data = [
            'mch_id' => $this->mch_id,
//            'media_hash' => hash_file('md5',$pic),
            'media_hash' => md5_file($pic),
        ];

        $data['sign'] = $this->getSignature($data);
        $data['media'] = curl_file_create($pic);

        $url = 'https://api.mch.weixin.qq.com/secapi/mch/uploadmedia';  //图片上传地址
        $postXml = $this->postCurl($url,$data);
        if($postXml === false){
            throw new \Exception($name.'上传图片失败');
        }
        $xmlObj = simplexml_load_string($postXml,'SimpleXMLElement', LIBXML_NOCDATA);

        $data = json_decode(json_encode($xmlObj),true);

        @unlink($pic);
        if(empty($data)){
            throw new \Exception($name.'图片上传过程中,微信数据解析失败');
        }
        if($data['return_code'] != 'SUCCESS'){
            throw new \Exception($name.$data['return_msg']);
        }

        if($data['result_code'] != 'SUCCESS'){
            throw new \Exception($name.$data['err_code_des']);
        }
        return $data['media_id'];
    }

    /**
     * 获取日志信息
     * @return string
     */
    public function getLogInfo(){
        return $this->log_info;
    }

    /**
     * 获取本地证书版本号
     * @return string
     * @throws \Exception
     */
    private function getKwySerial(){
        if(empty($this->serial_no)){
            throw new \Exception('本地证书版本号为空');
        }
        return $this->serial_no;
    }

    /**
     * 获取微信证书版本号
     * @return string
     * @throws \Exception
     */
    private function getWechatpaySerial(){
        if(empty($this->wechat_pay_serial)){
            throw new \Exception('微信证书版本号为空');
        }
        return $this->wechat_pay_serial;
    }

    /**
     * 获取渠道商本地证书秘钥文件路径
     * @return mixed
     * @throws \Exception
     */
    private function getChannelKeyPath(){
        if(empty($this->channel_key_path)){
            throw new \Exception('渠道商本地证书秘钥文件路径为空');
        }
        return $this->channel_key_path;
    }

    /**
     * 获取渠道商本地证书文件路径
     * @return mixed
     * @throws \Exception
     */
    private function getChannelCertPath(){
        if(empty($this->channel_cert_path)){
            throw new \Exception('渠道商本地证书文件路径为空');
        }
        return $this->channel_cert_path;
    }

    /**
     * 获取微信支付平台证书
     * @return mixed
     * @throws \Exception
     */
    private function getWechatPublicCert(){
        if(empty($this->wechat_public_cert)){
            throw new \Exception('微信支付平台证书为空');
        }
        return $this->wechat_public_cert;
    }

    /**
     * 获取请求通讯时的签名
     * @param $params
     * @param string $sign_type 签名类型: md5 | HMAC-SHA256
     * @return string
     */
    private function getSignature($params,$sign_type='md5'){
        ksort($params);
        $str = '';
        foreach($params as $k=>$v){
            $str .= $k.'='.$v.'&';
        }
        $str = $str."key=am2SpLslb3pwZZHWTpPEZwGlFdWxMws8";
        if($sign_type == 'HMAC-SHA256'){
            return strtoupper(hash_hmac('sha256', $str, 'am2SpLslb3pwZZHWTpPEZwGlFdWxMws8'));

        }else{
            return strtoupper(md5($str));
        }
    }

    /**
     * post请求
     * @param $url
     * @param $post_data
     * @return bool|mixed
     * @throws \Exception
     */
    private function postCurl($url,$post_data){
        $ch = curl_init();
        $header[] = "Accept-Charset: utf-8";
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );//强制ipv4解析
        curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (compatible; MSIE 5.01; Windows NT 5.0)');
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_AUTOREFERER, 1);

        //表单数据，是正规的表单设置值为非0
        curl_setopt($ch, CURLOPT_POST, 1);

        //设置证书
        //使用证书：cert 与 key 分别属于两个.pem文件
        $cert_path = getcwd().$this->getChannelCertPath();
        $key_path = getcwd().$this->getChannelKeyPath();
        curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM');
        curl_setopt($ch,CURLOPT_SSLCERT, $cert_path);
        curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');
        curl_setopt($ch,CURLOPT_SSLKEY, $key_path);

        curl_setopt($ch, CURLOPT_POSTFIELDS, $post_data);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $tmpInfo = curl_exec($ch);
        if (curl_errno($ch)) {
            curl_close($ch);
            return false;
        }else{
            curl_close($ch);
            return $tmpInfo;
        }
    }
}